<?php

namespace Drupal\search_api_fusion\Plugin\views\area;

use Drupal\Core\Url;
use Drupal\Core\Link;
use Drupal\Core\Path\CurrentPathStack;
use Drupal\views\Plugin\views\area\AreaPluginBase;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * Provides an area for spellcheck suggestions.
 *
 * @ViewsArea("search_api_fusion_spellcheck")
 *
 * @see Drupal\search_api_spellcheck\Plugin\views\area\DidYouMeanSpellCheck
 */
class FusionSpellcheck extends AreaPluginBase {

  /**
   * The current query parameters.
   *
   * @var array
   */
  protected $currentQuery;

  /**
   * The current path.
   *
   * @var \Drupal\Core\Path\CurrentPathStack
   */
  protected $currentPath;

  /**
   * Constructs a FusionSpellcheck object.
   *
   * @param array $configuration
   *   A configuration array containing information about the plugin instance.
   * @param string $plugin_id
   *   The plugin_id for the plugin instance.
   * @param mixed $plugin_definition
   *   The plugin implementation definition.
   * @param \Drupal\Core\Path\CurrentPathStack $current_path
   *   The current path.
   */
  final public function __construct(array $configuration, $plugin_id, $plugin_definition, CurrentPathStack $current_path) {
    parent::__construct($configuration, $plugin_id, $plugin_definition);
    $this->currentPath = $current_path;
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container, array $configuration, $plugin_id, $plugin_definition) {
    return new static(
      $configuration,
      $plugin_id,
      $plugin_definition,
      $container->get('path.current'),
    );
  }

  /**
   * Render the area.
   *
   * @param bool $empty
   *   (optional) Indicator if view result is empty or not. Defaults to FALSE.
   *
   * @return array
   *   In any case we need a valid Drupal render array to return.
   */
  public function render($empty = FALSE) {
    /** @var \Drupal\search_api\Plugin\views\query\SearchApiQuery */
    $query = $this->query;
    /** @var \Drupal\search_api\Query\ResultSetInterface */
    $results = $query->getSearchApiResults();

    // Return nothing if Fusion provided no spellcheck suggestion.
    $response = $results->getExtraData('search_api_solr_response');
    if (empty($response['spellcheck']['suggestions'])
      || !is_array($response['spellcheck']['suggestions'])) {
      return [];
    }

    // If Fusion provided spellcheck suggestions: order those by frequency.
    $suggestions = $response['spellcheck']['suggestions'];
    $word = NULL;
    foreach ($suggestions as $word => $data) {
      if (isset($data['suggestion'])) {
        usort($suggestions[$word]['suggestion'],
            function ($suggestion1, $suggestion2): int {
              return $suggestion1['freq'] <=> $suggestion2['freq'];
            });
      }
    }

    if (empty($suggestions[$word]['suggestion'])
      || !is_array($suggestions[$word]['suggestion'])) {
      return [];
    }

    // Extract suggestion with highest frequency.
    end($suggestions[$word]['suggestion']);
    $best_suggestion = current($suggestions[$word]['suggestion']);

    // Return hyperlink "Did you mean: <best spellcheck suggestion> ?".
    return [
      '#theme' => 'search_api_fusion_spellcheck',
      '#label' => $this->t('Did you mean:'),
      '#link' => $this->getSuggestionLink($best_suggestion['word']),
    ];
  }

  /**
   * Gets the suggestion link.
   *
   * @param string $suggestion
   *   The suggested search query.
   *
   * @return \Drupal\Core\Link
   *   The suggestion link.
   */
  protected function getSuggestionLink($suggestion) {
    if ($this->view->hasUrl()) {
      $path = '/' . $this->view->getPath();
    }
    else {
      $path = $this->currentPath->getPath($this->view->getRequest());
    }

    $currentQuery = $this->getCurrentQuery();

    unset($currentQuery['q']);
    $url = Url::fromUserInput($path, [
      'query' => [
        'query' => $suggestion,
      ] + $currentQuery,
    ]);

    return Link::fromTextAndUrl($suggestion, $url);
  }

  /**
   * Gets the current query parameters.
   *
   * @return array
   *   Key value of parameters.
   */
  protected function getCurrentQuery() {
    if (NULL === $this->currentQuery) {
      $this->currentQuery = $this->view->getRequest()->query->all();
    }
    return $this->currentQuery;
  }

}
