<?php

namespace Drupal\Tests\search_api_fusion\Unit;

use Drupal\Core\Path\PathValidatorInterface;
use Drupal\Core\StringTranslation\TranslationInterface;
use Drupal\search_api\Query\ResultSetInterface;
use Drupal\search_api\Plugin\views\query\SearchApiQuery;
use Drupal\search_api_fusion\Plugin\views\area\FusionSpellcheck;
use Drupal\Tests\UnitTestCase;
use Drupal\views\ViewExecutable;
use Symfony\Component\DependencyInjection\ContainerInterface;
use Symfony\Component\HttpFoundation\ParameterBag;
use Symfony\Component\HttpFoundation\Request;
use Drupal\Core\Path\CurrentPathStack;

/**
 * @coversDefaultClass \Drupal\search_api_fusion\Plugin\views\area\FusionSpellcheck
 * @group search_api_fusion
 */
class FusionSpellcheckTest extends UnitTestCase {

  /**
   * Mock of Drupal container object.
   *
   * @var \Symfony\Component\DependencyInjection\ContainerInterface|\PHPUnit\Framework\MockObject\MockObject
   */
  protected $container;
  /**
   * Mock of current path, used for testing.
   *
   * @var \Drupal\Core\Path\CurrentPathStack|\PHPUnit\Framework\MockObject\MockObject
   */
  protected $path;

  /**
   * FusionSpellcheck instance being tested.
   *
   * @var \Drupal\search_api_fusion\Plugin\views\area\FusionSpellcheck
   */
  protected $fusionSpellcheck;

  /**
   * Mock of result set.
   *
   * @var \Drupal\search_api\Query\ResultSetInterface[\Drupal\search_api\Item\ItemInterface]|\PHPUnit\Framework\MockObject\MockObject
   */
  protected $resultSet;

  /**
   * {@inheritdoc}
   */
  protected function setUp() : void {
    parent::setUp();

    $this->container = $this->getMockBuilder(ContainerInterface::class)
      ->disableOriginalConstructor()
      ->getMock();
    \Drupal::setContainer($this->container);
    $this->container->method('get')
      ->with('path.validator')
      ->willReturn($this->createStub(PathValidatorInterface::class));

    $this->path = $this->getMockBuilder(CurrentPathStack::class)
      ->disableOriginalConstructor()
      ->getMock();

    $this->path->method('getPath')
      ->willReturn('/search');

    $this->fusionSpellcheck = new FusionSpellcheck(['configuration array'], 'plugin_id', [], $this->path);

    $this->fusionSpellcheck->setStringTranslation($this->createStub(TranslationInterface::class));

    $this->fusionSpellcheck->query = $this->getMockBuilder(SearchApiQuery::class)
      ->disableOriginalConstructor()
      ->getMock();

    $this->resultSet = $this->getMockBuilder(ResultSetInterface::class)
      ->disableOriginalConstructor()
      ->getMock();

    $this->fusionSpellcheck->query
      ->method('getSearchApiResults')
      ->willReturn($this->resultSet);

    $this->fusionSpellcheck->view = $this->getMockBuilder(ViewExecutable::class)
      ->disableOriginalConstructor()
      ->getMock();

    $viewRequestStub = $this->getMockBuilder(Request::class)
      ->disableOriginalConstructor()
      ->getMock();
    $viewRequestStub->query = $this->getMockBuilder(ParameterBag::class)
      ->disableOriginalConstructor()
      ->getMock();
    $viewRequestStub->query->method('all')->willReturn([]);
    $this->fusionSpellcheck->view
      ->method('getRequest')
      ->willReturn($viewRequestStub);
  }

  /**
   * Test render spellcheck returns a suggestion with the highest frequency.
   */
  public function testRenderSpellcheckSuggestion(): void {
    $response['spellcheck']['suggestions'] = [
      'artz',
      [
        'suggestion' =>
          [
            ['word' => "arts", 'freq' => 39026],
            ['word' => "arth", 'freq' => 677],
            ['word' => "arti", 'freq' => 176],
            ['word' => "ariz", 'freq' => 90],
          ],
      ],
    ];

    $this->resultSet->method('getExtraData')
      ->willReturn($response);

    $result = $this->fusionSpellcheck->render();

    $this->assertStringContainsString('arts', $result['#link']->getText());
  }

  /**
   * Test render spellcheck when no suggestion was provided.
   */
  public function testRenderSpellcheckNoSuggestion() : void {
    $response['spellcheck']['suggestions'] = [];
    $this->resultSet->method('getExtraData')
      ->willReturn($response);

    $this->assertEmpty($this->fusionSpellcheck->render());
  }

  /**
   * Test render spellcheck when invalid suggestions are provided.
   */
  public function testRenderSpellcheckWhenSuggestionIsNotAnArray() : void {
    $response['spellcheck']['suggestions'] = 'arts';
    $this->resultSet->method('getExtraData')
      ->willReturn($response);

    $this->assertEmpty($this->fusionSpellcheck->render());
  }

  /**
   * Test render spellcheck when invalid suggestions are provided.
   */
  public function testRenderSpellcheckWhenSuggetionsDataHasNoSuggestions() : void {
    $response['spellcheck']['suggestions'] = [
      'tom',
      [
        'suggestion' => [],
      ],
    ];
    $this->resultSet->method('getExtraData')
      ->willReturn($response);

    $this->assertEmpty($this->fusionSpellcheck->render());
  }

}
